%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% make sure that qforms-tk is in your Matlab path
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% qforms-tk must be in your Matlab path and the demo directory
% has to be your current working directory

addpath /nfs/home2/berkes/del/qforms_tk/qforms
% cd MY_DEMO_PATH

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load the example quadratic form
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load unit4_quadratic_form

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute the optimal stimuli
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('*** compute the optimal stimuli\n');

% norm of the optimal stimuli
r = 1.8;
% tolerance for the computation of the optimal stimulus
tol = 1e-8;
% compute the optimal excitatory stimulus
xp = optimal_stimulus(H,f, r, tol);
% compute the optimal inhibitory stimulus
xm = optimal_stimulus(-H,-f, r, tol);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% show the optimal stimuli
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% size of an input patch
h = 16; w = 16;

clf; colormap(gray);
subplot(1,2,1);
imagesc(reshape(xp,h,w));
axis image; axis off;
title('x^+', 'FontSize', 24);

subplot(1,2,2);
imagesc(reshape(xm,h,w));
axis image; axis off;
title('x^-', 'FontSize', 24);
drawnow;

input('(press enter to continue)');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute the invariances in xp
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[V,d] = invariances(H,f, xp);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% show the first four invariances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% prepare figure
clf; colormap(gray);
% imagesc range
maxint = max(abs(xp));

% angular step in radians
dalpha = 2 *pi/180;
% angles
alpha_rg = 0:dalpha:pi/2;
alpha_rg = cat(2, alpha_rg, fliplr(alpha_rg), ...
	      -alpha_rg, fliplr(-alpha_rg));

% compute the value of the function at the maximum
out0 = 0.5*xp'*H*xp + f'*xp + c;
% minimal threshold value (80 percent of the maximum)
minout = 0.8*out0;
for i = 1:4,
  fprintf('*** show invariance %d\n', i);
  out = out0;

  for alpha = alpha_rg,
    % new stimulus
    x = cos(alpha)*xp + sin(alpha)*r*V(:,i);
    % response of the unit
    out = 0.5*x'*H*x + f'*x + c;

    if out > minout,
      % visualize x
      imagesc(reshape(x,h,w), [-maxint, maxint]);
      title(['invariance ' int2str(i)], 'FontSize', 24);
      axis image; axis off; drawnow;
    end
  end

  input('(press enter to continue)');
end
