function [B,T]=gram_schmidt(A)
% GRAM_SCHMIDT orthogonalization algorithm.
%   [B,T] = gram_schmidt(A) is a straightforward implementation of the
%   Gram-Schmidt orthogonalization algorithm. It computes a decomposition
%   of A as A=BT, B'B=I, where T is upper triangular.
%
%   The columns of B form an orthogonal basis that span the same subspace
%   as the columns of A.

  [m,n]=size(A);
  B=zeros(m,n);
  T=zeros(m,n);
  for j=1:n,
    for i=1:j-1,
      T(i,j)=B(:,i)'*A(:,j);
    end
    A(:,j)=A(:,j)-B(:,1:j-1)*T(1:j-1,j);
    T(j,j)=norm(A(:,j));
    B(:,j)=A(:,j)/T(j,j);
  end
